package orig.demo;

import orig.plotter.ColorPenPlotter;
import orig.plotter.PenColor;
import orig.plotter.Plotter;

public class DrawingProgram {
	public static void drawLine(final Plotter p, final int x1, final int y1,
			final int x2, final int y2) {
		p.penDown(false);
		p.moveTo(x1, y1);
		p.penDown(true);
		p.moveTo(x2, y2);
	}

	private static void drawE(final Plotter p, final int x, final int y,
			final int scale) {
		// Draw an E
		drawLine(p, x, y, 2 * scale + x, y);
		drawLine(p, x, y, x, 4 * scale + y);
		drawLine(p, x, 4 * scale + y, 2 * scale + x, 4 * scale + y);
		drawLine(p, x, 2 * scale + y, scale + x, 2 * scale + y);
	}

	private static void drawN(final Plotter p, final int x, final int y,
			final int scale) {
		// Draw an N
		drawLine(p, x, 4 * scale + y, x, y);
		drawLine(p, x, y, x + scale * 2, 4 * scale + y);
		drawLine(p, x + scale * 2, 4 * scale + y, x + scale * 2, y);
	}

	private static void drawS(final Plotter p, final int x, final int y,
			final int scale) {
		// Draw an S
		drawLine(p, x + scale * 2, y, x, y);
		drawLine(p, x, y, x, 2 * scale + y);
		drawLine(p, x, 2 * scale + y, x + 2 * scale, 2 * scale + y);
		drawLine(p, x + 2 * scale, 2 * scale + y, x + 2 * scale, 4 * scale + y);
		drawLine(p, x + 2 * scale, 4 * scale + y, x, 4 * scale + y);
	}

	private static void drawV(final Plotter p, final int x, final int y,
			final int scale) {
		// Draw a V
		drawLine(p, x, y, scale + x, 4 * scale + y);
		drawLine(p, scale + x, 4 * scale + y, scale * 2 + x, y);
	}

	public static void drawingExample(final Plotter p) {
		p.home();

		// Quite bad ordering...
		// Add an E
		p.setPenColor(PenColor.RED);
		drawE(p, 10, 10, 10);

		// And an N
		p.setPenColor(PenColor.BLUE);
		drawN(p, 70, 10, 10);

		// And an S
		p.setPenColor(PenColor.RED);
		drawS(p, 10, 60, 10);

		// Draw a V
		p.setPenColor(PenColor.BLACK);
		drawV(p, 40, 10, 10);

		// Much nicer ordering, unlikely to be optimal.
		// Add an E
		p.setPenColor(PenColor.RED);
		drawE(p, 110, 10, 10);

		// And an S
		p.setPenColor(PenColor.RED);
		drawS(p, 110, 60, 10);

		// Draw a V
		p.setPenColor(PenColor.BLACK);
		drawV(p, 140, 10, 10);

		// And an N
		p.setPenColor(PenColor.BLUE);
		drawN(p, 170, 10, 10);
	}

	// Nice looking function with terrible performance.
	public static void verySad(final Plotter p) {
		p.setPenColor(PenColor.RED);
		long t = System.nanoTime();
		for (int i = 0; i < 20; i++) {
			drawLine(p, 0, i * 10, i, i * 10 + 2);
			drawLine(p, 300, i * 10, 300 + i, i * 10 + 2);
		}
		System.out.println("Time taken for verySad: " + (System.nanoTime() - t)
				/ 1000000000.0 + "s");
	}

	// Very non-nice function but is likely to do its drawing
	// in a fashion that's optimal with regards to time.
	public static void veryNice(final Plotter p) {
		p.setPenColor(PenColor.BLUE);
		long t = System.nanoTime();
		for (int i = 0; i < 20; i++) {
			if (i % 2 == 1) {
				drawLine(p, 10, i * 10, 10 + i, i * 10 + 2);
			} else {
				drawLine(p, 10 + i, i * 10 + 2, 10, i * 10);
			}
		}
		for (int i = 19; i >= 0; i--) {
			if (i % 2 == 1) {
				drawLine(p, 310, i * 10, 310 + i, i * 10 + 2);
			} else {
				drawLine(p, 310 + i, i * 10 + 2, 310, i * 10);
			}
		}
		System.out.println("Time taken for veryNice: "
				+ (System.nanoTime() - t) / 1000000000.0 + "s");
	}

	public static void drawCircle(final Plotter p, final int centreX,
			final int centreY,
			final int radius) {
		double stepLength = 1.0 / radius * 300;

		p.penDown(false);
		p.moveTo(centreX + radius, centreY);
		p.penDown(true);
		for (double i = 0; i < 360; i += stepLength) {
			double rad = Math.toRadians(i);
			p.moveTo((int) (Math.cos(rad) * radius + centreX), (int) (Math
					.sin(rad)
					* radius + centreY));
		}
		p.moveTo(centreX + radius, centreY);
	}

	private static void randomTree(final Plotter p, final int x, final int y,
			final double scale,
			final int degrees, final double shrinkage, final int rotate,
			final double threshold,
			final double randDegrees) {
		if (scale > threshold) {
			double rad = Math.toRadians(degrees);
			int newX = (int) (x + Math.cos(rad) * scale);
			int newY = (int) (y + Math.sin(rad) * scale);
			drawLine(p, x, y, newX, newY);
			randomTree(
					p,
					newX,
					newY,
					scale * shrinkage,
					(int) (degrees + rotate + Math.random() * randDegrees - randDegrees / 2.0),
					shrinkage, rotate, threshold, randDegrees);
			randomTree(
					p,
					newX,
					newY,
					scale * shrinkage,
					(int) (degrees - rotate + Math.random() * randDegrees - randDegrees / 2.0),
					shrinkage, -rotate, threshold, randDegrees);
		}
	}

	public static void randomTree(final Plotter p, final int x, final int y,
			final double scale,
			final double shrinkage, final int rotate, final double threshold,
			final double randDegrees) {
		randomTree(p, x, y, scale, -90, shrinkage, rotate, threshold,
				randDegrees);
	}

	private static void threeCircles(final Plotter p, final double x,
			final double y, final double radius) {
		drawCircle(p, (int) x, (int) y, (int) radius);
		final double scale = 0.4472;
		if (radius > 10) {
			for (int i = 0; i < 3; i++) {
				double rad = Math.toRadians(i * 120 + radius);
				double newX = Math.sin(rad) * radius * 0.57 + x;
				double newY = Math.cos(rad) * radius * 0.57 + y;
				threeCircles(p, newX, newY, radius * scale);
			}
		}
	}

	private static void snowFlake(final Plotter p, final double x,
			final double y, final double radius, final double degOffset) {
		if (radius > 2) {
			for (int i = 0; i < 5; i++) {
				double rad = Math.toRadians(i * 72 + degOffset);
				double newX = Math.sin(rad) * radius + x;
				double newY = Math.cos(rad) * radius + y;
				drawLine(p, (int) x, (int) y, (int) newX, (int) newY);
				snowFlake(p, newX, newY, radius * 0.42, degOffset + 5);
			}
			drawCircle(p, (int) x, (int) y, (int) radius);
		}
	}

	public static void main(final String[] args) {
		Plotter p = new ColorPenPlotter(600, 500, 999, 999, 30);
		p.penDown(true);
		veryNice(p);
		verySad(p);
		drawingExample(p);
		for (int i = 10; i < 100; i += 10) {
			drawCircle(p, 50 + i, 400, i);
		}
		p.setPenColor(PenColor.BLACK);
		randomTree(p, 300, 500, 100, 0.7, 25, 6.0, 15.0);
		p.setPenColor(PenColor.GREEN);
		snowFlake(p, 300, 250, 150, 5);
		p.setPenColor(PenColor.PURPLE);
		threeCircles(p, 300, 250, 200);
	}
}
