package orig.demo;

public class GridBagConstraintsAlternative {
	private final int gridx;
	private final int gridy;
	private final int gridwidth;
	private final int gridheight;

	// Ignoring the other 7 parameters for the sake of brevity...

	private GridBagConstraintsAlternative(final Builder builder) {
		this.gridx = builder.gridx;
		this.gridy = builder.gridy;
		this.gridwidth = builder.gridwidth;
		this.gridheight = builder.gridheight;
	}

	public static class Builder {
		private final int gridx; // Required parameter
		private final int gridy; // Required parameter
		// Set defaults here.
		private int gridwidth = 0;
		private int gridheight = 0;

		// And so on and so forth.

		public Builder(final int gridx, final int gridy) {
			this.gridx = gridx;
			this.gridy = gridy;
		}

		public Builder gridWidth(final int gridWidth) {
			this.gridwidth = gridWidth;
			return this;
		}

		public Builder gridHeight(final int gridHeight) {
			this.gridheight = gridHeight;
			return this;
		}

		public GridBagConstraintsAlternative build() {
			return new GridBagConstraintsAlternative(this);
		}
	}

	public static void main(final String[] args) {
		// We want to build an object that sets the height but takes the default
		// value for the width.
		GridBagConstraintsAlternative gbc = new Builder(1, 2).gridHeight(2)
				.build();

		// Yay! We did not need to create bogus default values and we ran no
		// risk of getting the order of parameters wrong.
	}
}
