package orig.demo;

import java.util.EnumMap;
import java.util.Map;

import orig.plotter.PenColor;
import orig.plotter.Plotter;

public class InstrumentedPlotterAdapter implements Plotter {
	private final Map<PenColor, Double> usedInk;
	private final Plotter plotter;
	private PenColor color;
	private int x;
	private int y;
	private boolean penDown;
	private double travelledDistance;

	private static final int GUESSED_DIMENSION = 500;

	public InstrumentedPlotterAdapter(final Plotter plotter) {
		// What risk(s) are we running when we don't make a defensive copy of
		// plotter?
		// Hint: What happens when the class that created the plotter doesn't
		// let
		// go of the reference and manipulates the plotter by other means than
		// through this class?
		this.plotter = plotter;

		this.usedInk = new EnumMap<PenColor, Double>(PenColor.class);

		// Reset all used inks.
		for (PenColor c : PenColor.values()) {
			this.usedInk.put(c, Double.valueOf(0.0));
		}

		this.travelledDistance = 0;

		// For x and y we have no idea where pen-head is. This is as a good
		// guess
		// as any other.
		this.x = GUESSED_DIMENSION;
		this.y = GUESSED_DIMENSION;
	}

	public double getTravelledDistance() {
		return this.travelledDistance;
	}

	@SuppressWarnings("boxing")
	public double getUsedInk(final PenColor c) {
		return this.usedInk.get(c);
	}

	@Override
	public void home() {
		// Make a guess at the current position.
		this.travelledDistance += distance(0, 0, this.x, this.y);
		this.x = 0;
		this.y = 0;
		this.plotter.home();
	}

	@SuppressWarnings("boxing")
	@Override
	public void moveTo(final int x1, final int y1) {
		double d = distance(this.x, this.y, x1, y1);
		this.travelledDistance += d;
		this.plotter.moveTo(x1, y1);
		if (this.penDown) {
			// Increase the used ink based on the distance from the current
			// point.
			this.usedInk.put(this.color, this.usedInk.get(this.color) + d);
		}
		this.x = x1;
		this.y = y1;
	}

	@Override
	public void penDown(final boolean v) {
		this.penDown = v;
		this.plotter.penDown(v);
	}

	@Override
	public void setPenColor(final PenColor color) {
		// Assume we go to the origin, <0, 0>, to fetch a new pen.
		this.travelledDistance += distance(this.x, this.y, 0, 0);
		this.plotter.setPenColor(color);
	}

	private double distance(final int x1, final int y1, final int x2,
			final int y2) {
		return Math.hypot(Math.abs(x1 - x2), Math.abs(y1 - y2));
	}

}
