package orig.screen;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;

import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 * A very simple class for drawing lines and circles in a window.
 * 
 * @author evensen
 *
 */

public class ScreenDrawer {
	private Graphics g;

	/**
	 * Creates a new window with a drawing area.
	 * 
	 * @param width
	 *            The width of the paper/window in pixels.
	 * @param height
	 *            The height of the paper/window in pixels.
	 */
	public ScreenDrawer(int width, int height) {
		JFrame f = new JFrame("ScreenDrawer");
		JPanel p = new JPanel(true);
		p.setMinimumSize(new Dimension(width, height));
		p.setPreferredSize(p.getMinimumSize());
		p.setVisible(true);
		p.setOpaque(true);
		f.add(p);
		f.pack();
		f.setVisible(true);
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		g = p.getGraphics();
		p.setBackground(Color.WHITE);
		g.setColor(Color.BLACK);
		g.drawLine(0, 0, width, height);
		g.drawLine(0, height, width, 0);
	}

	/**
	 * Draws a line from <code>p1</code> to <code>p2</code> with the colour
	 * <code>col</code>
	 * 
	 * @param p1
	 *            Point to draw from
	 * @param p2
	 *            Point to draw to
	 * @param col
	 *            Colour to use for drawing
	 */
	public void drawLine(Point p1, Point p2, Color col) {
		g.setColor(col);
		g.drawLine(p1.x, p1.y, p2.x, p2.y);
	}

	/**
	 * Draws a circle centred at <code>p</code> with radius <code>radius</code> using colour <code>col</code>.
	 * 
	 * @param p
	 *            Centre of circle
	 * @param radius
	 *            The circle's radius
	 * @param col
	 *            Colour to use for drawing
	 */
	public void drawCircle(Point p, int radius, Color col) {
		g.setColor(col);
		g.drawOval(p.x - radius, p.y - radius, radius, radius);
	}

}
