package calendarapp.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.GregorianCalendar;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import calendarapp.plugins.Pluggable;
import calendarapp.plugins.PluginLoader;


/**
 * This class is a view of a calendar month and is used by the Calendar class
 * which makes a 6x4 grid of CalendarMonth widgets.
 * @author adamw
 *
 */
@SuppressWarnings("serial")
public class CalendarMonth extends JPanel {
	private GregorianCalendar startDate;

	public CalendarMonth(GregorianCalendar startDate) {
		this.startDate = startDate;

		setLayout(new GridBagLayout());
		setBorder(BorderFactory.createMatteBorder(1, 1, 1, 1, Color.gray));

		addMonthDescriptor();
		addDayLabels();
		addDays();
	}

	private void addMonthDescriptor() {
		GridBagConstraints c = new GridBagConstraints();

		c.weighty = 1.0;
		c.weightx = 1.0;
		c.fill = GridBagConstraints.BOTH;
		c.gridy = 0;
		c.gridwidth = 7;

		add(new MonthLabel(startDate), c); 
	}

	private void addDayLabels() {
		String[] days = {"Su", "Mo", "Tu", "We", "Th", "Fr", "Sa",
		                 "Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"};
		GridBagConstraints c = new GridBagConstraints();

		c.weighty = 0.5;
		c.weightx = 1.0;
		c.fill = GridBagConstraints.BOTH;
		c.gridy = 1;
		c.insets = new Insets(10, 0, 3, 0);
		c.gridwidth = 1;

		for (int i = 0; i < 7; i++) {
			JLabel label = new JLabel(days[startDate.getFirstDayOfWeek() + i - 1]);
			label.setFont(new Font("SansSerif", Font.BOLD, 9));
			label.setHorizontalAlignment(SwingConstants.CENTER);
			add(label, c);
		}
	}

	private void addDays() {
		int gridy =  2;
		List<Pluggable<?>> plugins = PluginLoader.loadCalendarPlugins();
		int startDay =  startDate.get(GregorianCalendar.DAY_OF_WEEK) >= startDate.getFirstDayOfWeek() ?
		                startDate.get(GregorianCalendar.DAY_OF_WEEK) - startDate.getFirstDayOfWeek() :
		                7 - startDate.get(GregorianCalendar.DAY_OF_WEEK);
		int endDay = startDate.getActualMaximum(GregorianCalendar.DAY_OF_MONTH);

		GridBagConstraints constraints;
		Component component;

		for (int i = 0; i < 7*6; i++) {
			if (i % 7 == 0)
				gridy++;

			if (i < startDay || i > endDay + startDay - 1) {
				constraints = CalendarDayButton.getGlobalConstraints();
				component = new JPanel();
			} else {
				GregorianCalendar date = (GregorianCalendar) startDate.clone();
				date.set(GregorianCalendar.DAY_OF_MONTH, i - startDay + 1);

				component = new CalendarDayButton(Integer.toString(i - startDay + 1), date, plugins);
				constraints = ((CalendarDayButton) component).getConstraints();
				
			}

			constraints.gridy = gridy;
			add(component, constraints);
		}
	}
}
