package edu.chl.hajo.monopoly.core;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.List;

import edu.chl.hajo.monopoly.constants.StateChanges;
import edu.chl.hajo.monopoly.core.space.AbstractSpace;

/**
 * Representing the overall game A singleton
 * 
 * @author hajo
 * 
 */
public class Game implements IGame  {
	private final PropertyChangeSupport observers = new PropertyChangeSupport(
			this);

	private List<Player> players;
	private final Board board = new Board();
	private Player actualPlayer;
	private Dices dices = new Dices();

	private static IGame instance;

	public static IGame getInstance() {
		if (instance == null) {
			instance = new Game();
		}
		return instance;
	}

	private Game() {
	}

	// ------ IGame ---------------------------

	/*
	 * Players must be in correct order
	 */
	public void init(List<Player> players) {
		this.players = players;
		actualPlayer = players.get(0);
	}

	@Override
	public void switchActualPlayer() {
		int index = players.indexOf(actualPlayer);
		actualPlayer = players.get(++index % players.size());
		observers.firePropertyChange(
				StateChanges.GAME_NEW_ACTUAL_PLAYER.toString(), null,
				actualPlayer.getName());
	}
	
	
	@Override
	public AbstractSpace getActualSpace() {
		int pos = actualPlayer.getPosition();
		return board.getSpaces().get(pos);
	}

	@Override
	public List<AbstractSpace> getSpaces() {
		return board.getSpaces();
	}

	@Override
	public List<Player> getPlayers() {
		return players;
	}

	@Override
	public Player getActualPlayer() {
		return actualPlayer;
	}
	
	@Override
	public Dices getDices() {
		return dices;
	}

	// ----- IObservable ------------

	@Override
	public void addObserver(PropertyChangeListener observer) {
		observers.addPropertyChangeListener(observer);

	}

	
}
