package edu.chl.hajo.monopoly.core.space;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.HashSet;
import java.util.Set;

import edu.chl.hajo.monopoly.constants.StateChanges;
import edu.chl.hajo.monopoly.core.Player;
import edu.chl.hajo.monopoly.util.IObservable;


/**
 * 
 * Base class for all streets, railroads, corners, jail, etc..
 * 
 * @author hajo
 * 
 */
public abstract class AbstractSpace implements IObservable {

	protected final PropertyChangeSupport observers = new PropertyChangeSupport(
			this);
	// All spaces must have a unique name
	private String name;
	// Players on this space
	private Set<Player> visitingPlayers = new HashSet<Player>();

	public AbstractSpace(String name) {
		this.name = name;
	}

	/*
	 * Adding removing players common for all spaces
	 */
	public void addVisitingPlayer(Player p) {
		visitingPlayers.add(p);
		observers.firePropertyChange(
				StateChanges.SPACE_ADD_VISITING.toString(), null,
				p.getPiece());
	}

	public void removeVisitingPlayer(Player p) {
		visitingPlayers.remove(p);
		observers.firePropertyChange(
				StateChanges.SPACE_REMOVE_VISITING.toString(), null,
				p.getPiece());
	}

	public boolean isVisitingPlayer(Player p) {
		return visitingPlayers.contains(p);
	}

	public String getName() {
		return name;
	}

	// ----- IObservable ---------------

	@Override
	public void addObserver(PropertyChangeListener observer) {
		observers.addPropertyChangeListener(observer);
	}
}
