package edu.chl.hajo.monopoly.view;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashMap;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.BevelBorder;

import edu.chl.hajo.monopoly.constants.Piece;
import edu.chl.hajo.monopoly.constants.StateChanges;
import edu.chl.hajo.monopoly.core.space.AbstractSpace;
import edu.chl.hajo.monopoly.util.IObservable;

/**
 * Common code for Panels
 */
@SuppressWarnings("serial")
public abstract class AbstractSpacePanel extends JPanel implements
		PropertyChangeListener {
	protected JLabel jLlbName;
	private Map<Piece, JLabel> map = new HashMap<Piece, JLabel>();
	protected IObservable observable;

	protected final MouseAdapter listener = new MouseAdapter() {

		@Override
		public void mouseClicked(MouseEvent e) {
			showDetails();
		}
	};

	// To be overridden by subclasses
	protected abstract void showDetails();

	public AbstractSpacePanel() {
		// TODO UGLY fix how players are displayed
		map.put(Piece.CAR, new JLabel(Piece.CAR.toString()));
		map.put(Piece.DOG, new JLabel(Piece.DOG.toString()));
		map.put(Piece.IRON, new JLabel(Piece.IRON.toString()));
		map.put(Piece.TOP_HAT, new JLabel(Piece.TOP_HAT.toString()));
		initGUI();
		this.addMouseListener(listener);
	}

	public AbstractSpacePanel(AbstractSpace space) {
		this();
		space.addObserver(this);
		jLlbName.setText(space.getName());
	}

	@Override
	public void propertyChange(PropertyChangeEvent evt) {
		String name = evt.getPropertyName();
		// TODO remove
		System.out.println(name);
		if (StateChanges.SPACE_ADD_VISITING.toString().equals(name)) {
			Piece p = (Piece) evt.getNewValue(); // TODO Safe??
			this.add(map.get(p));
		} else if (StateChanges.SPACE_REMOVE_VISITING.toString().equals(name)) {
			Piece p = (Piece) evt.getNewValue(); // TODO Safe??
			this.remove(map.get(p));
		}
		repaint();
	}

	private void initGUI() {
		try {
			{
				this.setPreferredSize(new Dimension(80, 80));
				this.setBorder(BorderFactory
						.createBevelBorder(BevelBorder.LOWERED));
				{
					jLlbName = new JLabel();
					this.add(jLlbName);
					jLlbName.setText("JLblName");
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void paintComponent(Graphics g) {
		super.paintComponent(g);
	}

}
