package edu.chl.hajo.monopoly;

import java.util.ArrayList;
import java.util.List;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import edu.chl.hajo.monopoly.constants.Piece;
import edu.chl.hajo.monopoly.core.Game;
import edu.chl.hajo.monopoly.core.IGame;
import edu.chl.hajo.monopoly.core.Player;
import edu.chl.hajo.monopoly.core.space.AbstractSpace;
import edu.chl.hajo.monopoly.core.space.Street;
import edu.chl.hajo.monopoly.ctrl.BuyPropertyCtrl;
import edu.chl.hajo.monopoly.ctrl.EndRoundCtrl;
import edu.chl.hajo.monopoly.ctrl.MoveCtrl;


public class TestCtrl {

	IGame g;
	
	@Before
	// Run before each test method
	public void setUpBefore() throws Exception {
		List<Player> p = new ArrayList<Player>();
		// Ordering matters! Use buy game
		p.add(new Player("Olle", Piece.DOG));
		p.add(new Player("Fia", Piece.TOP_HAT));
		p.add(new Player("Pelle", Piece.CAR));
		p.add(new Player("Sanna", Piece.IRON));
		g =  Game.getInstance();
		((Game)g).init(p);
	}
	
	@Test
	public void testMove() {
		//Go to community
		 new MoveCtrl(2).execute();
		//Go to community
		
		Player p = g.getActualPlayer();
		AbstractSpace s = g.getActualSpace();
		
		Assert.assertTrue( s.isVisitingPlayer(p));
		Assert.assertTrue( s.getName().equals("community"));
		Assert.assertTrue( p.getName().equals("Olle"));		
	}
	
	@Test
	public void testEndRound(){
		Player p = g.getActualPlayer();
		new EndRoundCtrl().execute();
		new EndRoundCtrl().execute();
		new EndRoundCtrl().execute();
		new EndRoundCtrl().execute();
		
		Assert.assertTrue( g.getActualPlayer() == p );		
	}
	
	@Test
	public void testBuyProperty(){
		Player p = g.getActualPlayer();
		
		Assert.assertTrue(p.getName().equals("Olle"));
				
		// Know this is a street (i.e. a property)
		Street s = (Street) g.getSpaces().get(3);
		int oldBalance = p.getBalance();
		// Go to street
		new MoveCtrl(3).execute();
		new BuyPropertyCtrl(s).execute();
		
		Assert.assertTrue( s.getOwner() == p);
		Assert.assertTrue( p.getProperties().size() == 1);
		Assert.assertTrue( p.getBalance() < oldBalance);		
	}

}
