package edu.chl.hajo.monopoly.view;

import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.LineBorder;

import edu.chl.hajo.monopoly.core.IPlayer;
import edu.chl.hajo.monopoly.event.GameEvents;
import edu.chl.hajo.monopoly.event.StateChanges;
import edu.chl.hajo.monopoly.util.Options;
import edu.chl.hajo.monopoly.util.PlayerData;
import edu.chl.hajo.monopoly.view.details.PlayerDetails;

/**
 * View of Player. This (and the child PlayerDetails) are readonly
 * dont need any reference to player object (uses PlayerData).
 * 
 * NOTE: This is dynamic GUI must be connected to EventBus (in MainFrame)
 * 
 * @author hajo
 * 
 */
@SuppressWarnings("serial")
public class PlayerPanel extends JPanel implements PropertyChangeListener {
	private JLabel jLabel1;

	private JLabel jLblBalance;
	private JLabel jLabel5;
	private JLabel jLblInJail;
	private JLabel jLabel3;
	private JLabel jLblName;

	private PlayerData playerData;
	private final LineBorder activeBorder = new LineBorder(Color.RED, 2, false);
	private final LineBorder passiveBorder = new LineBorder(
			new java.awt.Color(0, 0, 0), 1, false);
	private LineBorder actualBorder = passiveBorder;

	private transient MouseAdapter listener = new MouseAdapter() {

		@Override
		public void mouseClicked(MouseEvent e) {
			new PlayerDetails(PlayerPanel.this, playerData).setVisible(true);
		}
	};

	public PlayerPanel(IPlayer player) {
		this.playerData = player.data();
		this.addMouseListener(listener);
		this.setBackground(Options.getColorForPiece(playerData.piece));
		initGUI();
		initData();
	}

	private void initData() {
		jLblBalance.setText(String.valueOf(playerData.balance));
		jLblName.setText(playerData.name);
	}

	@Override
	public void propertyChange(PropertyChangeEvent evt) {
		String s = evt.getPropertyName();

		if (s.equals(StateChanges.PLAYER_WITHDRAW.toString())) {
			jLblBalance.setText(String.valueOf(evt.getNewValue()));
		} else if (s.equals(StateChanges.PLAYER_ADD_INCOME.toString())) {
			jLblBalance.setText(String.valueOf(evt.getNewValue()));
		} else if (s.equals(StateChanges.GAME_NEW_ACTUAL_PLAYER.toString())) {
			boolean b = (Boolean) evt.getNewValue();
			if( b ){
				actualBorder = activeBorder;			
			}else{
				actualBorder = passiveBorder;			
			}
			this.setBorder(actualBorder);
			repaint();
		} else if (s.equals(StateChanges.PLAYER_IN_JAIL.toString())) {
			Boolean b = (Boolean) evt.getNewValue();
			jLblInJail.setText(b.toString());
		} else if (s.equals(GameEvents.PLAYER_DIE.toString())) {
			// TODO test
			JPanel parent = (JPanel) this.getParent();
			parent.remove(this);
			parent.repaint();
		}
	}

	private void initGUI() {
		try {
			{
				GridLayout thisLayout = new GridLayout(3, 2);
				thisLayout.setHgap(5);
				thisLayout.setVgap(5);
				thisLayout.setColumns(2);
				thisLayout.setRows(5);
				this.setLayout(thisLayout);
				this.setBorder(actualBorder);
				{
					jLabel1 = new JLabel();
					this.add(jLabel1);
					jLabel1.setText("Name");
				}
				{
					jLblName = new JLabel();
					this.add(jLblName);
					jLblName.setText("jLblName");
				}
				{
					jLabel3 = new JLabel();
					this.add(jLabel3);
					jLabel3.setText("Balance");
				}
				{
					jLblBalance = new JLabel();
					this.add(jLblBalance);
					jLblBalance.setText("jLblBalance");
				}
				{
					jLabel5 = new JLabel();
					this.add(jLabel5);
					jLabel5.setText("InJail");
				}
				{
					jLblInJail = new JLabel();
					this.add(jLblInJail);
					jLblInJail.setText("false");
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

}
