package edu.chl.hajo.monopoly.core;

import java.util.ArrayList;
import java.util.List;

import edu.chl.hajo.monopoly.constants.GameEvents;
import edu.chl.hajo.monopoly.constants.Piece;
import edu.chl.hajo.monopoly.constants.StateChanges;
import edu.chl.hajo.monopoly.core.space.Property;
import edu.chl.hajo.monopoly.event.EventBus;
import edu.chl.hajo.monopoly.util.Config;


/**
 * A player in the game
 * 
 * @author hajo
 * 
 */
public class Player  {

	private final String name;
	private final Piece piece; 
	private int position = 0;
	private boolean inJail = false;
	private int balance = Config.getPlayerStartBalance();
	private List<Property> properties = new ArrayList<Property>();
	private EventBus eb = EventBus.getInstance();

	public Player(String name, Piece piece) {
		super();
		this.name = name;
		this.piece = piece;
	}

	public int getPosition() {
		return position;
	}

	public void moveTo(int position) {
		this.position = position;
	}

	public int getBalance() {
		return balance;
	}
	
	public void setBalance(int balance) {
		this.balance = balance;
	}

	public String getName() {
		return name;
	}
	
	public Piece getPiece() {
		return piece;
	}
	/*
	 * No checks here. See ctrl.ForceWithDraw
	 */
	public void withDraw(int amount) {
		balance -= amount;
		eb.send(this, StateChanges.PLAYER_WITHDRAW, null, balance);
	}

	public void addIncome(int income) {
		balance += income;
		eb.send(this, StateChanges.PLAYER_ADD_INCOME, null, balance);
	}

	
	public void die() {
		//TODO eb.send(this, GameEvents.PLAYER_DIE, null, this);
	}

	public void inJail(boolean b) {
		this.inJail = b;
		eb.send(this, StateChanges.PLAYER_IN_JAIL, null, b);
	}

	public boolean isInJail() {
		return inJail;
	}

	public void addProperty(Property property) {
		properties.add(property);
	}

	public void removeProperty(Property property) {
		properties.remove(property);
	}

	public List<Property> getProperties() {
		return properties;
	}

	public void mustSellToPay(int amount) {
		eb.broadcast(GameEvents.PLAYER_MUST_SELL, null, amount);	
	}
	
}
