package edu.chl.hajo.monopoly.view;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.SwingUtilities;

import edu.chl.hajo.monopoly.build.GameBuilder;
import edu.chl.hajo.monopoly.core.Dices;
import edu.chl.hajo.monopoly.core.IPanelVisitor;
import edu.chl.hajo.monopoly.core.space.AbstractSpace;
import edu.chl.hajo.monopoly.core.space.NonProperty;
import edu.chl.hajo.monopoly.core.space.Property;
import edu.chl.hajo.monopoly.core.space.Street;
import edu.chl.hajo.monopoly.core.space.Tax;
import edu.chl.hajo.monopoly.event.EventBus;
import edu.chl.hajo.monopoly.view.dialog.DialogHandler;

/**
 * Building the GUI and connect static components to EventBus.
 * Building of and connecting of (static) GUI handled outside the GUI
 * @author hajo
 *
 */
public class GUIBuilder {

	private MainFrame mainFrame;
	private GameBuilder gameBuilder;
	
	public GUIBuilder(GameBuilder gameBuilder) {
		this.gameBuilder = gameBuilder;
	}

	public MainFrame build()
			throws InvocationTargetException, InterruptedException {
		// Will block until GUI created
		SwingUtilities.invokeAndWait(new Runnable() {
			public void run() {
				List<AbstractSpace> spaces = gameBuilder.getSpaces();
				List<AbstractSpacePanel> spacePanels = buildSpacePanels(spaces);
				Dices dices = gameBuilder.getDices();
				DicesPanel dicesPanel = new DicesPanel(dices);
				ControlPanel ctrPanel = new ControlPanel();
				mainFrame = new MainFrame(spacePanels, dicesPanel, ctrPanel);
				new DialogHandler(mainFrame);
				/*
				 * Connecting
				 */
				EventBus eb = EventBus.getInstance();
				eb.connect(mainFrame);
				eb.connect(dicesPanel);
				eb.connect(ctrPanel);
			}
		});
		return mainFrame;
	}
	
	private List<AbstractSpacePanel> buildSpacePanels(List<AbstractSpace> spaces) {
		List<AbstractSpacePanel> spacePanels = new ArrayList<AbstractSpacePanel>();

		/*
		 * Using the visitor pattern
		 */
		PanelVisitor pf = new PanelVisitor();
		for (AbstractSpace s : spaces) {
			s.visit(pf);
			AbstractSpacePanel asp = pf.getPanel();
			spacePanels.add(asp);
			/*
			 * Every visible space has a direct connection to
			 * the space class in model
			 */
			EventBus.getInstance().connect(asp, s);
		}
		return spacePanels;
	}

	// Used by Visitor pattern
	private static final class PanelVisitor implements IPanelVisitor {
		AbstractSpacePanel sp;

		public AbstractSpacePanel getPanel() {
			return sp;
		}

		@Override
		public void setPanel(Property p) {
			sp = new PropertyPanel(p);
		}

		@Override
		public void setPanel(NonProperty p) {
			sp = new NonPropertyPanel(p);
		}

		@Override
		public void setPanel(Street p) {
			sp = new StreetPanel(p);
		}

		@Override
		public void setPanel(Tax p) {
			sp = new NonPropertyPanel(p);
		}
	}
}
