package edu.chl.hajo.monopoly.gui;

import edu.chl.hajo.monopoly.core.Board;
import edu.chl.hajo.monopoly.core.IMonopoly;
import edu.chl.hajo.monopoly.core.MonopolyOptions;
import edu.chl.hajo.monopoly.core.Player;
import edu.chl.hajo.monopoly.core.space.Property;
import edu.chl.hajo.monopoly.core.space.Space;
import edu.chl.hajo.monopoly.event.Event;
import edu.chl.hajo.monopoly.event.EventBus;
import edu.chl.hajo.monopoly.event.IEventHandler;
import edu.chl.hajo.monopoly.gui.dialogs.SpaceDialog;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.HashMap;
import java.util.Map;
import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;

/**
 * View of all the Spaces
 * @author hajo
 */
public class SpacesPanel extends JPanel implements IEventHandler {

    // View of a Single Space
    private class SpacePanel extends JPanel {

        private final JLabel name = new JLabel();
        private final JLabel owner = new JLabel();
        private final JList visitors = new JList();
        private final DefaultListModel<String> model = new DefaultListModel<>();
        private final Space space;    
        
        public SpacePanel(Space s) {
            this.space = s;
            this.setLayout(new GridLayout(5, 1));
            this.setBorder(BorderFactory.createLineBorder(Color.BLUE));

            name.setText(s.getName());
            if (s instanceof Property) {  //Ugly
                Property p = (Property) s;
                owner.setText(p.getOwner().getName());
                this.add("owner", owner);
            }
            this.add("name", name);
            this.add("visitors", visitors);
            this.setSize(40, 40);
            this.addMouseListener(mouseListener);

        }
        MouseAdapter mouseListener = new MouseAdapter() {

            @Override
            public void mouseClicked(MouseEvent e) {
                new SpaceDialog(MainFrame.getInstance(), name.getText(), 
                        true, monopoly).setVisible(true);
            }
        };

    };

    public SpacesPanel() {
        this.setLayout(new GridLayout(1, MonopolyOptions.getNSpaces()));
        EventBus.INSTANCE.register(this);
    }
    
    private IMonopoly monopoly; 
    @Override
    public void onEvent(Event evt) {
        switch (evt.getTag()) {
            case NEW_GAME:
                monopoly = (IMonopoly) evt.getValue();
                newGame(monopoly.getBoard());
                for (Player p : monopoly.getPlayers()) {
                    putPlayer(p);
                }
                break;
            case ADD_PLAYER:
                Player actual = (Player) evt.getValue();
                putPlayer(actual);
                break;
            case REMOVE_PLAYER:
                actual = (Player) evt.getValue();
                removePlayer(actual);
                break;
             case BUY:
                 actual = (Player) evt.getValue();
                 SpacePanel sp = map.get(actual.getPosition());
                 sp.owner.setText(actual.getName());
                 break;
            default: ;
        }
    }

    // Given a Space (from model) lookup it's view
    private final Map<Space, SpacePanel> map = new HashMap<>();

    private void newGame(Board b) {
        for (Space s : b) {
            SpacePanel sp = new SpacePanel(s);
            map.put(s, sp);
            this.add(sp);
        }
    }

    private void putPlayer(Player p) {
        SpacePanel sp = map.get(p.getPosition());
        sp.model.addElement(p.getPiece().toString());
        sp.visitors.setModel(sp.model);
    }

    private void removePlayer(Player p) {
        SpacePanel sp = map.get(p.getPosition());
        sp.model.removeElement(p.getPiece().toString());
        sp.visitors.setModel(sp.model);
    }

}
